from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_joined_game_server import UserJoinedGameServer
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from modules.permission.service.permission_service import PermissionService


class UserJoinedGameServerMutation(QueryABC):
    def __init__(
        self,
        logger: LoggerABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        user_joined_game_servers: UserJoinedGameServerRepositoryABC,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        permissions: PermissionService,
    ):
        QueryABC.__init__(self, "UserJoinedGameServerMutation")

        self._logger = logger
        self._servers = servers
        self._users = users
        self._user_joined_game_servers = user_joined_game_servers
        self._bot = bot
        self._db = db
        self._permissions = permissions

        self.set_field("userJoined", self.resolve_user_joined)
        self.set_field("userLeaved", self.resolve_user_leaved)

    def resolve_user_joined(self, *_, input: dict):
        user = self._users.get_user_by_id(input["userId"])
        self._can_user_mutate_data(user.server, UserRoleEnum.admin)

        active = self._user_joined_game_servers.find_active_user_joined_game_server_by_user_id(user.user_id)
        if active is not None:
            self._logger.debug(
                __name__,
                f"Skip UserJoinedGameServer for user {user.user_id}. User already plays on {active.game_server}.",
            )
            return

        new = UserJoinedGameServer(user, input["gameServer"], input["joinedOn"])
        self._user_joined_game_servers.add_user_joined_game_server(new)
        self._db.save_changes()

        return self._user_joined_game_servers.get_active_user_joined_game_server_by_user_id(user.user_id)

    def resolve_user_leaved(self, *_, input: dict):
        user = self._users.get_user_by_id(input["userId"])
        self._can_user_mutate_data(user.server, UserRoleEnum.admin)

        active = self._user_joined_game_servers.get_active_user_joined_game_server_by_user_id(user.user_id)
        active.leaved_on = input["leavedOn"]

        self._user_joined_game_servers.update_user_joined_game_server(active)
        self._db.save_changes()

        return active
